/*
Copyright (C) 2011 by Blazing Games Inc.

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
*/
package com.blazinggames.cgl.atlas
{
	import flash.display.BitmapData;
	import flash.events.Event;
	import flash.geom.Point;
	import flash.geom.Rectangle;

	/**
	 * Holds information about Image Clips. Image clips are rectangular 
	 * sections of a particular page in the parent Image Atlas.
	 *  
	 * @author Billy D. Spelchan
	 */	
	public class ClipInfo
	{
// ***** CONSTANTS *****
		
		public static const ORIGIN_STRINGS:Array = new Array(
			"top left", "top center", "top right",
			"middle left", "middle center", "middle right",
			"bottom left", "bottom center", "bottom right",
			"relative" );
		public static const ORIGIN_TOP_LEFT:int = 0;
		public static const ORIGIN_TOP_CENTER:int = 1;
		public static const ORIGIN_TOP_RIGHT:int = 2;
		public static const ORIGIN_MIDDLE_LEFT:int = 3;
		public static const ORIGIN_MIDDLE_CENTER:int = 4;
		public static const ORIGIN_MIDDLE_RIGHT:int = 5;
		public static const ORIGIN_BOTTOM_LEFT:int = 6;
		public static const ORIGIN_BOTTOM_CENTER:int = 7;
		public static const ORIGIN_BOTTOM_RIGHT:int = 8;
		public static const ORIGIN_RELATIVE:int = 9;

		public static const TOP_LEFT_POINT:Point = new Point(0,0);
		
// ***** VARIABLES *****

		private var _page:PageInfo;
		private var _rect:Rectangle;
		private var _clipID:String;
		private var _origin:Point;
		private var _originType:int;
		private var _bitmapData:BitmapData;
		
// ***** CONSTRUCTOR *****
		
		/**
		 * Creates an image clip.
		 *  
		 * @param name name of the image clip.
		 * @param page name of the page the image clip gets the image from.
		 * @param rect the clipping rectangle.
		 * @param origin where, relative to the clip, the origin is
		 * @param originType type of origin
		 */		
		public function ClipInfo(	name:String="pixel", 
								 	page:PageInfo = null,
									rect:Rectangle = null,
									origin:Point = null, originType:int = ORIGIN_TOP_LEFT
								)
		{
			_page = page;
			_rect = rect;
			if (_rect == null)
				_rect = new Rectangle(0,0,1,1);
			_clipID = name;
			_origin = origin;
			if (_origin == null)
				_origin = new Point(0,0);
			_originType = originType;
			_bitmapData = null;
		}
		
// ***** METHODS - XML Handling *****
		
		/**
		 * Clip tag format <CLIP id='clip id' page='page id' 
		 * x='x' y='y' w='width' h='height' 
		 * originX='origin x' originY='origin y' originType='origin type' />
		 * 
		 * @param xml The XML data to be converted into an image clip
		 * @param atlas the source image atlas
		 * @param addToAtlas should clip be added to the atlas clip list.
		 */
		public function setFromXML(xml:XML, atlas:ImageAtlas, addToAtlas:Boolean = true):void
		{
			var pageID:String = xml.@page;
			_clipID = xml.@id;
			_page = atlas.getPage(pageID);
			_rect.x = parseInt(xml.@x);
			_rect.y = parseInt(xml.@y);
			_rect.width = parseInt(xml.@w);
			_rect.height = parseInt(xml.@h);
			_origin.x = parseInt(xml.@originX);
			_origin.y = parseInt(xml.@originY);
			_originType = 0;
			var tempLen:int = ORIGIN_STRINGS.length;
			var s:String = xml.@originType;
			for (var cntr:int = 0; cntr < tempLen; ++cntr) {
				if (ORIGIN_STRINGS[cntr] == s) {
					_originType = cntr;
					break;
				}
			}
			if (addToAtlas) {
				atlas.addClip(this);
			}
		}

		/**
		 * Converts the image clip into an XML string. This can be useful for
		 * debugging or for tools that manually create clips.
		 *  
		 * @return String with image clip tag.
		 */
		public function toXMLString():String
		{
			var page:String = "error";
			if (_page != null)
				page = _page.pageName;
			return "<CLIP id=\'"+_clipID+
				"\' page=\'"+page+
				"\' x=\'"+_rect.x+
				"\' y=\'"+_rect.y+
				"\' w=\'"+_rect.width+
				"\' h=\'"+_rect.height+
				"\' originX=\'"+_origin.x+
				"\' originY=\'"+_origin.y+
				"\' originType=\'"+ORIGIN_STRINGS[_originType]+"\' />";
		}

// ***** METHODS - Getters and Setters *****
		
		/**
		 * The page to grab the clip from 
		 * @return The name of the source Image Atlas page
		 */		
		public function get sourcePage():PageInfo
		{
			return _page;
		}
		
		/**
		 * Sets the image atlas page that the clip is taken from.
		 *  
		 * @param value Image Atlas page to clip from
		 */
		public function set sourcePage(value:PageInfo):void
		{
			_page = value;
		}

		/**
		 * returns the source rectangle being clipped.
		 * 
		 * @return  the source rectangle being clipped.
		 */		
		public function get sourceRect():Rectangle
		{
			return _rect;
		}

		/**
		 * Sets the clipping rectangle 
		 * @param value rectangle to be clipped.
		 */		
		public function set sourceRect(value:Rectangle):void
		{
			_rect = value;
		}

		/**
		 * returns the name of the clip. 
		 * 
		 * @return the name of the clip.
		 */		
		public function get clipID():String
		{
			return _clipID;
		}

		/**
		 * Sets the name of the clip. Should be unique, or at least unique to
		 * the image atlas containing the clip.
		 *  
		 * @param value name of the clip. 
		 */		
		public function set clipID(value:String):void
		{
			_clipID = value;
		}

		/**
		 * The origin point relative to the top-left of the clip 
		 * 
 		 * @return The origin of the clip
		 */
		public function get origin():Point
		{
			return _origin;
		}

		/**
		 * Sets the origin of the clip. Origin is relative to the
		 * top left of the clip.
		 *  
		 * @param value the point of the origin.
		 */		
		public function set origin(value:Point):void
		{
			_origin = value;
		}

		/**
		 * The type of the origin is not really useful for coding but can be
		 * of some use for any tools using the image atlas library.
		 *  
		 * @return type of origin the origin value represents.
		 */
		public function get originType():int
		{
			return _originType;
		}

		/**
		 * sets the type of origin that the origin point represents. 
		 * 
		 * @param value type of origin that the origin point represents.
		 */		
		public function set originType(value:int):void
		{
			_originType = value;
		}

		/**
		 * returns the bitmap data that represents this image clip. If the 
		 * page containing the clip is not yet loaded, it will be loaded in
		 * the background.
		 * 
		 * @return the BitmapData containing the image clip.
		 */		
		public function get bitmapData():BitmapData
		{
			if (_bitmapData == null) {
				_bitmapData = new BitmapData(_rect.width, _rect.height, true, 0xFF00FF);
				if (_page != null) {
					_page.addEventListener(PageInfo.PAGE_LOADED_EVENT, onPageLoaded);
					_page.loadPage();
					//_bitmapData.copyPixels(_page.bitmapData, _rect, TOP_LEFT_POINT);
				}
			}
			return _bitmapData;
		}
		
// ***** METHODS - Event Handlers *****		
		
		/**
		 * Internal method for filling out the bitmap data once the page that 
		 * is to be clipped has been loaded.
		 *  
		 * @param event the PageInfo PAGE_LOADED_EVENT
		 */
		protected function onPageLoaded(event:Event):void
		{
			_bitmapData.copyPixels(_page.bitmapData, _rect, TOP_LEFT_POINT);
			_page.removeEventListener(PageInfo.PAGE_LOADED_EVENT, onPageLoaded);
		}

	}
}