/*
Copyright (C) 2011 by Blazing Games Inc.

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
*/
package com.blazinggames.cgl.atlas
{
	import flash.geom.Point;

	/**
	 * Hold a description of a frame of animation.
	 * 
	 * @author Billy D. Spelchan
	 */
	public class FrameInfo
	{
// ***** CONSTANTS *****
		
		public static const FRAMERATE_20FPS:int = 50;
		public static const FRAMERATE_25FPS:int = 40;
		public static const FRAMERATE_30FPS:int = 33;
		public static const FRAMERATE_50FPS:int = 20;
		public static const FRAMERATE_60FPS:int = 17;
		
// ***** VARIABLES *****
		
		private var _frameNumber:int;
		private var _clip:ClipInfo;
		private var _offset:Point;
		private var _duration:int;
		
// ***** CONSTRUCTOR *****
		
		/**
		 * Creates a new frame.
		 *  
		 * @param n the frame number.
		 * @param clip the clip that represents the frame image.
		 * @param offset Any additional offset information for the frame.
		 * @param duration how long the frame should be shown in milli-seconds.
		 */		
		public function FrameInfo(n:int = 0,
								  clip:ClipInfo = null, 
								  offset:Point = null,
									duration:int = FRAMERATE_30FPS)
		{
			_frameNumber = n;
			_clip = clip;
			_offset = offset;
			if (_offset == null)
				_offset = new Point();
			_duration = duration;
		}

// ***** METHODS - XML Handling *****
		
		/**
		 * Frame tags are children of the Animation tag and have the format 
		 * <FRAME number='frame' clip='clip id' x='offset x' y='offset y' duration='duration' /> 
		 * 
		 * @param xml the xml data to be parsed
		 * @param atlas the source Image Atlas
		 */
		public function setFromXML(xml:XML, atlas:ImageAtlas):void
		{
			_frameNumber = xml.@number;
			_clip = atlas.getClip(xml.@clip);
			_offset.x = parseInt(xml.@x);
			_offset.y = parseInt(xml.@y);
			_duration = xml.@duration;
		}

		/**
		 * Converts the frame information into a string representation of the
		 * XML tag.
		 *  
		 * @return String representation of the XML tag.
		 */		
		public function toXMLString():String
		{
			var clipID:String = "error";
			if (_clip != null)
				clipID = _clip.clipID;
			return "<FRAME number=\'"+_frameNumber+
				"\' clip=\'"+clipID+
				"\' x=\'"+_offset.x+
				"\' y=\'"+_offset.y+
				"\' duration=\'"+_duration+"\' />";
		}

// ***** METHODS - Getters and Setters *****
		
		/**
		 * returns the frame number this frame represents.
		 * 
		 * @return the frame number this frame represents. 
		 */
		public function get frameNumber():int
		{
			return _frameNumber;
		}

		/**
		 * Sets the frame number.
		 *  
		 * @param value frame number this frame represents.
		 */		
		public function set frameNumber(value:int):void
		{
			_frameNumber = value;
		}

		/**
		 * returns the image clip info for this frame.
		 *  
		 * @return the image clip info for this frame.
		 */		
		public function get clip():ClipInfo
		{
			return _clip;
		}

		/**
		 * Sets the image clip info used to represent the frame. 
		 * @param value the image clip to use for this frame.
		 */		
		public function set clip(value:ClipInfo):void
		{
			_clip = value;
		}

		/**
		 * The relative offset of the frame.
		 * 
		 * @return The relative offset of the frame.
		 */		
		public function get offset():Point
		{
			return _offset;
		}

		/**
		 * The offset allows a frame to be nudged. This can be useful in cases 
		 * where the frames are following (centered on) an object but the 
		 * playback should actually move. Walking animations, for example.
		 * 
		 * @param value the offset of the frame.
		 */		
		public function set offset(value:Point):void
		{
			_offset = value;
		}

		/**
		 * How long in milli-seconds the frame should be shown.
		 *  
		 * @return the duration in milli-seconds.
		 */		
		public function get duration():int
		{
			return _duration;
		}

		/**
		 * How long in milli-seconds the frame should be shown.
		 * 
		 * @param value the duration in milli-seconds.
		 */
		public function set duration(value:int):void
		{
			_duration = value;
		}


	}
}