/*
Copyright (C) 2011 by Blazing Games Inc.

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
*/
package com.blazinggames.cgl.atlas
{
	import flash.events.EventDispatcher;
	import flash.events.IEventDispatcher;
	import flash.geom.Rectangle;
	
	/**
	 * The Image Atlas class is essentially a resource manager for images.
	 * The atlas can contain multiple pages with any number of clips taking
	 * chunks of those pages and any number of animation sequences using
	 * clips for frames. This is the heart of the image atlas animation 
	 * library. 
	 * 
	 * @author Billy D. Spelchan
	 */	
	public class ImageAtlas extends EventDispatcher
	{
// ***** VARIABLES *****
		
		protected var pages:Array;
		protected var clips:Array;
		protected var animations:Array;

// ***** CONSTRUCTOR *****
		
		/**
		 * Creates an image atlas.
		 * 
		 * @param target 
		 */		
		public function ImageAtlas(target:IEventDispatcher=null)
		{
			super(target);
			pages = new Array();
			clips = new Array();
			animations = new Array();
		}
		
// ***** METHODS - XML Handling *****
		
		/**
		 * Fills the atlas with data from Image Atlas tag 
		 * 	<IMAGEATLAS>
		 * 		<!-- page tags as described in PageInfo class -->
		 * 		<!-- clip tags as described in ClipInfo class -->
		 * 		<!-- animation tags as described in AnimationInfo class -->
		 * 		<IMAGESTRIP baseName='name' page='pageID' x='x' y='y' w='width' h='height' 
		 * 			frames='number of frames' frameRate='durration in milliseconds' />
		 *  </IMAGEATLAS>
		 *  
		 * @param xml source xml file to build atlas from
		 * @param load should pages be loaded immediately
		 */
		public function setFromXML(xml:XML, load:Boolean = true):void
		{
			var list:XMLList = xml.elements("PAGE");
			var cntr:int;
			var len:int = list.length();
			var pageInfo:PageInfo;
			for (cntr = 0; cntr < len; ++cntr) {
				pageInfo = new PageInfo();
				pageInfo.setFromXML(xml.PAGE[cntr], load);
				addPage(pageInfo);
			}

			list = xml.elements("CLIP");
			len = list.length();
			var clipInfo:ClipInfo;
			for (cntr = 0; cntr < len; ++cntr) {
				clipInfo = new ClipInfo();
				clipInfo.setFromXML(xml.CLIP[cntr], this, true);
			}

			list = xml.elements("ANIMATION");
			len = list.length();
			var animInfo:AnimationInfo;
			for (cntr = 0; cntr < len; ++cntr) {
				animInfo = new AnimationInfo();
				animInfo.setFromXML(xml.ANIMATION[cntr], this);
				addAnimation(animInfo);
			}

			list = xml.elements("IMAGESTRIP");
			len = list.length();
			for (cntr = 0; cntr < len; ++cntr) {
				setImageStripFromXML(xml.IMAGESTRIP[cntr]);
			}

		}
		
		/**
		 * parses an image strip tag and calls addImageStrip with contents.
		 * <IMAGESTRIP baseName='name' page='pageID' x='x' y='y' w='width' h='height' 
		 * 			frames='number of frames' frameRate='durration in milliseconds' />
		 *  </IMAGEATLAS>
		 *  
		 * @param xml source xml file to build image strip from
		 */
		public function setImageStripFromXML(xml:XML):void
		{
			var baseName:String = xml.@baseName;
			var pageID:String = xml.@page;
			var x:int = parseInt(xml.@x);
			var y:int = parseInt(xml.@y);
			var width:int = parseInt(xml.@w);
			var height:int = parseInt(xml.@h);
			var frames:int = parseInt(xml.@frames);
			var frameRate:int = parseInt(xml.@frameRate);
			
			addImageStrip(baseName, pageID, x, y, width,height, frames, frameRate);
		}
		
		/**
		 * Converts the image atlas into an xml string. This is great for 
		 * tools as the image atlas can be manually manipulated and when done
		 * saved as an XML file.
		 *  
		 * @return Image Atlas as an XML-formatted String.
		 */		
		public function toXMLString():String
		{
			var s:String = "<IMAGEATLAS>\n";
			var cntr:int;
			var len:int = pages.length;
			for (cntr = 0; cntr < len; ++cntr)
				s = s + pages[cntr].toXMLString() + "\n";
			len = clips.length;
			for (cntr = 0; cntr < len; ++cntr)
				s = s + clips[cntr].toXMLString() + "\n";
			len = animations.length;
			for (cntr = 0; cntr < len; ++cntr)
				s = s + animations[cntr].toXMLString() + "\n";
			s=s+"</IMAGEATLAS>";
			
			return s;
		}
		
// ***** METHODS - Page support *****
		
		/**
		 * Adds the indicated page to the image atlas. 
		 * 
		 * @param page page to be added to the atlas.
		 */		
		public function addPage(page:PageInfo): void
		{
			pages.push(page);
		}

		/**
		 * retrieves a page from the image atlas.
		 *  
		 * @param id name of page to be retrieved.
		 * @return page information for desired page.
		 */
		public function getPage(id:String):PageInfo
		{
			var ret:PageInfo = null;
			var page:PageInfo;
			
			for (var cntr:int=0; cntr < pages.length; ++cntr) {
				page = PageInfo(pages[cntr]);
				if (page.pageName == id)
					ret = page;
			}
			
			return ret;
		}

		/**
		 * Checks to see if a page is in the Image Atlas.
		 *  
		 * @param id name of page to check for
		 * @return true if page is in atlas, otherwise false.
		 */		
		public function isPage(id:String):Boolean
		{
			return (getPage(id) != null);
		}

		/**
		 * Removes the indicated page from the image atlas.
		 *  
		 * @param page Page to be removed.
		 */		
		public function removePage(page:PageInfo):void
		{
			var indx:int = -1;
			var pi:PageInfo;
			
			for (var cntr:int = 0; cntr < pages.length; ++cntr) {
				pi = pages[cntr];
				if (page.pageName == pi.pageName) {
					indx == cntr;
					break;
				}
			}
			
			if (indx >= 0)
				pages.splice(indx, 1);
		}
		
// ***** METHODS - Clip support *****

		/**
		 * Adds an image clip to the image atlas.
		 *  
		 * @param c clip to be added to atlas.
		 */		
		public function addClip(c:ClipInfo): void
		{
			clips.push(c);
		}
		
		/**
		 * retrieves a clip from the image atlas.
		 *  
		 * @param clipID name of clip to be retrieved.
		 * @return  the desired clip or null if not found.
		 */		
		public function getClip(clipID:String):ClipInfo
		{
			var ret:ClipInfo = null;
			var clip:ClipInfo;
			
			for (var cntr:int = 0; cntr < clips.length; ++cntr) {
				clip = ClipInfo(clips[cntr]);
				if (clip.clipID == clipID)
					ret = clip;
			}
			
			return ret;
		}

		/**
		 * Checks to see if a particular image clip is in the atlas.
		 *  
		 * @param id name of the image clip to find
		 * @return true if image clip found, otherwise false.
		 */		
		public function isClip(id:String):Boolean
		{
			return (getClip(id) != null);
		}
		
		/**
		 * Removes an image clip from the image atlas.
		 *  
		 * @param clip clip to be removed.
		 */
		public function removeClip(clip:ClipInfo):void
		{
			var indx:int = -1;
			var ci:ClipInfo;
			
			for (var cntr:int = 0; cntr < clips.length; ++cntr) {
				ci = clips[cntr];
				if (clip.clipID == ci.clipID) {
					indx == cntr;
					break;
				}
			}
			
			if (indx >= 0)
				clips.splice(indx, 1);
		}

// ***** METHODS - Animation support *****
		
		/**
		 * Adds an animation sequence to the image atlas.
		 *  
		 * @param a animation sequence to add.
		 */		
		public function addAnimation(a:AnimationInfo): void
		{
			animations.push(a);
		}

		/**
		 * retrieves an animation sequence from the atlas.
		 *  
		 * @param sequenceName name of sequence to be retrieved.
		 * @return sequence or null if not found.
		 */		
		public function getAnimation(sequenceName:String):AnimationInfo
		{
			var ret:AnimationInfo = null;
			var anim:AnimationInfo;
			
			var len:int = animations.length;
			for (var cntr:int = 0; cntr < len; ++cntr) {
				anim = AnimationInfo(animations[cntr]);
				if (anim.sequenceName == sequenceName)
					ret = anim;
			}
			
			return ret;
		}

		/**
		 * Checks to see if an animation sequence is in the atlas.
		 *  
		 * @param sequenceName name of sequence to check for.
		 * @return true if in atlas, otherwise false.
		 */		
		public function isAnimation(sequenceName:String):Boolean
		{
			return (getAnimation(sequenceName) != null);
		}

		/**
		 * Removes an animation sequence from the atlas.
		 * 
		 * @param animation sequence to be removed from atlas.
		 */		
		public function removeAnimation(animation:AnimationInfo):void
		{
			var indx:int = -1;
			var anim:AnimationInfo;
			
			var len:int = animations.length;
			for (var cntr:int = 0; cntr < len; ++cntr) {
				anim = AnimationInfo(animations[cntr]);
				if (animation.sequenceName == anim.sequenceName) {
					indx == cntr;
					break;
				}
			}
			
			if (indx >= 0)
				animations.splice(indx, 1);
		}

		/**
		 * Adds a horizontal strip of image clips to the library. This is a
		 * convienience function as it allows the creation of a number of 
		 * image clips, frames and an animation sequence with a single call.
		 * The clips will have the base name followed by the frame number in
		 * square brackets. The animation will be named the base name.
		 *  
		 * @param baseName Name of the resulting animation sequence
		 * @param pageID Source page for the image strip
		 * @param x first frame x coordinate
		 * @param y first frame y coordinate
		 * @param width width of each frame
		 * @param height height of each frame
		 * @param frames number of frames in the animation
		 * @param frameRate duration of each frame.
		 * 
		 * @return returns the animation that this image strip represents.
		 */
		public function addImageStrip( baseName:String, pageID:String,
									   x:int, y:int, width:int, height:int,
									   frames:int, frameRate:int = 33):AnimationInfo
		{
			var anim:AnimationInfo = new AnimationInfo();
			anim.sequenceName = baseName;
			var curX:int = x;
			var curY:int = y;
			var clip:ClipInfo;
			var pageInfo:PageInfo = getPage(pageID);
			for (var cntr:int = 0; cntr < frames; ++cntr) {
				clip = new ClipInfo(baseName+"["+cntr+"]",pageInfo,
						new Rectangle(curX, curY, width, height));
				curX += width;
				addClip(clip);
				anim.addFrame(new FrameInfo(cntr, clip, null, frameRate));
			}
			addAnimation(anim);
			return anim; 
		}

	}
}
